# This is a -*- Tcl -*- library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# code.tcl --- Tcl Code Processing
#
# Copyright (C) 1997, 1998 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0.4
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package require Sequence
package require Flow

# ---------------------------------------------------------------------
# PROCEDURE: juf_safe_eval [OPTIONS] SCRIPT ...
#
# Evaluate each SCRIPT as Tcl script in a temporary safe interpreter.
#
# Options are:
# -aliases 		NAME	Create a alias for each key in array NAME.
# -exit         COMMAND Calls COMMAND with interpreter as argument
#                       after processing SCRIPT.
# -interp		NAME	Store command for safe interpreter within NAME.
# -nosafe				Use ordinary slave interpreter.
# -renames		NAME	Array NAME with commands to rename. 
# -stats		NAME	Fill array NAME with statistics.
# -variables	NAME	Set a variable for each key in array NAME.
# -unknown		NAME	Procedure for unknown functions.
# --					End of options.
# ---------------------------------------------------------------------

proc juf_safe_eval {args} {
	global errorInfo
	#
	# Process options
	#
	set safe 1
	set cont 1
	set exithandler ""
	while {$cont} {
		juf_branch [lindex $args 0] -aliases {
			upvar [Juf::Sequence::shift args 2] aliases
		} -exit {
			set exithandler [Juf::Sequence::shift args 2]
		} -interp {
			upvar [Juf::Sequence::shift args 2] interp
		} -nosafe {
			set safe 0; Juf::Sequence::shift args
		} -renames {
			upvar [Juf::Sequence::shift args 2] renames
		} -stats {
			upvar [Juf::Sequence::shift args 2] stats
		} -unknown {
			set unknown [Juf::Sequence::shift args 2]
		} -variables {
			upvar [Juf::Sequence::shift args 2] variables
		} -- {
			Juf::Sequence::shift args; set cont 0
		} default {
			set cont 0
		}
	}
	#
	# Create safe interpreter, rename commands and create aliases
	#
	if $safe {
		set interp [interp create -safe]
	} else {
		set interp [interp create]
	}
	foreach cmdname [array names renames] {
		$interp eval rename $cmdname $renames($cmdname)
	}
	foreach cmdname [array names aliases] {
		eval {$interp} alias {$cmdname} $aliases($cmdname)
	}
	if [info exists unknown] {
		eval $interp alias unknown $unknown $interp
	}
	#
	# Create initial variables
	#
	foreach varname [array name variables] {
		$interp eval set [list $varname] [list $variables($varname)]
	}
	#
	# Execute commands in script within safe interpreter
	#
	set result ""
	foreach script $args {
		if [catch "$interp eval {$script}" result] {
			interp delete $interp
			error $result $errorInfo
		}
	}
	#
	# Assemble information if requested
	#
	if [string length [info vars stats]] {
		array set stats [list cmdcount [$interp eval info cmdcount]]
	}
	#
	# Call exit handler if desired
	#
	if [string length $exithandler] {
		$exithandler $interp
	}
	#
	# Delete save interpreter
	#
	interp delete $interp
	return $result
}

# ------------------------------------------------------------
# PROCEDURE: juf_safe_source [OPTIONS] FILE
#
# Evaluate FILE as Tcl script in a temporary safe interpreter.
# See `juf_safe_eval' for explanations of the options.
# ------------------------------------------------------------

proc juf_safe_source {args} {
	global errorInfo
	#
	# Read script from file
	#
	set fd [open [Juf::Sequence::pop args] r]
	if [catch "read $fd" script] {
		close $fd
		error $script $errorInfo
	}
	close $fd
	uplevel juf_safe_eval $args [list $script]
}

package provide juf::code 0.0.1
package provide Code 0.0.4
