# This is a -*- Tcl -*- library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# file.tcl --- Manipulating files and filenames
#
# Copyright (C) 1996, 1997, 1998 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0.4
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package require Sequence

# ------------------------------------------------------------------
# PROCEDURE: juf_file_expand NAME [DIR]
#
# Makes absolute filename from NAME and performs tilde substitution.
# If NAME is a relative filename, it is located in the current
# working directory or in DIR if given. This function does not
# (yet) remove ".." or "." in filenames.
# ------------------------------------------------------------------

proc juf_file_expand {name {dir ""}} {
	#
	# absolutify
	#
	if [string compare $dir ""] {
		set fabs [file join $dir $name]
	} else {
		set fabs [file join [pwd] $name]
	}
	#
	# split in parts and perform tilde substitution
	# (see Tcl's file manpage)
	#
	set dlist [file split $fabs]
	set dlist [lreplace $dlist 0 0 [file dirname [lindex $dlist 0]] \
			[file tail [lindex $dlist 0]]]
	#
	# O.K., lbnl merge together
	#
	eval file join $dlist
}

# ------------------------------------------------------------------
# PROCEDURE: juf_file_strip NAME1 NAME2
#
# Returns NAME1 stripped of the common part of both NAME1 and NAME2.
# ------------------------------------------------------------------

proc juf_file_strip {name1 name2} {
	set dlist1 [file split $name1]
	set dlist2 [file split $name2]

	for {set i 0} "\$i < [llength $dlist1]" {incr i} {
		#
		# Grab leading file name components and compare them
		#
		set dir1 [Juf::Sequence::shift dlist1]
		set dir2 [Juf::Sequence::shift dlist2]
		if {[string compare $dir1 $dir2]} {
			#
			# Different directory name => assemble result
			#
			return [eval file join {$dir1} $dlist1]
		}
	}
	if [llength $dlist1] {
		eval file join $dlist1
	}
}

# ------------------------------
# PROCEDURE: juf_file_class FILE
#
# Returns class for FILE.
# ------------------------------

proc juf_file_class {file} {
	set type [file type $file]
	#
	# only regular files should be examined further
	#
	if [string compare $type file] {return $type}
	#
	# run Unix `file' command
	#
	set msg [exec file $file]
	#
	# extract class
	#
	set list [juf_split -- $msg : 2]
	set list [juf_split -- [lindex $list 1] , 2]

	switch -- [string trim [lindex $list 0]] {
		"exported SGML document text" {return sgml}
		"gzip compressed data" {return compressed}
	}
}

# ---------------------------------------------------------
# juf_file_slurp - reads complete file into single variable
#
# in:	file	name of file
#		name	name of variable
#
# out:  1 in case of success, 0 otherwise
# ---------------------------------------------------------

proc juf_file_slurp {file name} {
	upvar $name v_buf
	#
	# open file
	#
	if {[catch "open $file" v_chl] != 0} {
		Juf::Error::error $file [Juf::Error::tclmsg]
		return 0
	}
	#
	# read contents
	#
	if {[catch "read $v_chl" v_buf] != 0} {
		Juf::Error::error $file [Juf::Error::tclmsg]
		return 0
	}
	close $v_chl
	return 1
}

# --------------------------------------------------------------
# juf_file_mkdirs_aux - auxiliary function for juf_file_mkdirs
# --------------------------------------------------------------

proc juf_file_mkdirs_aux {dir} {
	if {$dir == "/"} {return 1}
	if {! [juf_file_mkdirs_aux [file dirname $dir]]} {return 0}
	if [file exists $dir] {
		return 1
	} else {
		if {[catch "exec mkdir $dir"] != 0} {
			err_fat $dir 
		}		
		return 1
	}
}

# -------------------------------------------------------------------------
# juf_file_mkdirs - make directories 
#
# in:	$args	list of directories
# out:	1 if successful, 0 otherwise
#
# Creates any missing parent directories for each argument.
# Do not consider an argument directory that already exists to be an error.
# -------------------------------------------------------------------------

proc juf_file_mkdirs {args} {
	foreach dir $args {
		if {! [juf_file_mkdirs_aux $dir]} {
			return 0
		}
	}
}

# -------------------------------------------------
# juf_file_iscwd - checks if filename corresponds
# 					to current working directory
#
# in:	$name 	filename to be checked
# out:  1 if successful, 0 otherwise
# -------------------------------------------------

proc juf_file_iscwd {name} {
	if {[file isdir $name]} {
		set cwd [pwd]	;# remember current working directory
		cd $name		;# move to proposed directory
		if {[pwd] == $cwd} {
			return 1
		} else {
			cd $cwd		;# restore working directory
			return 0
		}
	} else {
		return 0
	}
}

package provide juf::file 0.0.4
