# This is a -*- Tcl -*- library file and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# getopts.tcl --- options processing module
#
# Copyright (C) 1996, 1997, 1998 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0.1
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package require Error

# ======================================================================
# User Options
#
# The variables `juf_getopts_initskip' and `juf_getopts_argdescrpos'
# control the formatting of the option listing.
# Each line will be indented `juf_getopts_initskip' characters. The
# argument description will be placed at the Nth column 
# (N = value of `juf_getopts_argdescrpos'), possibly on a single line.
# ======================================================================

set juf_getopts_initskip 2
set juf_getopts_argdescrpos 31

# ---------------------------------------------------
# juf_getopts - extract options from command line
#
# in:	$optspecs	list with option specifications
#		$optarr		array for options storage
#		$newargv	remaining arguments 
#		$oldargv	command line arguments
#
# Changes global variable $argv and array $opts
# ---------------------------------------------

proc juf_getopts { optspecs optarr newargv oldargv } {
	upvar $optarr opts	
	upvar $newargv resargv 
	set options('') ""
	unset options('')
	set refoptions('') ""
	unset refoptions('') 
	set new_argv ""

	foreach optspec $optspecs {
		set spec [lindex $optspec 0]

		# Look for argument specifiers
		# ============================

		if [regexp {^([^=:]+)(([=:])([s]))?$} $spec dummy optnames dummy argre argtype] {
			set optnames [split $optnames "|"]

			# replace dashes with underscores
			# ===============================

			regsub -all -- - [lindex $optnames 0] _ optmain

			# Generate entry for option array
			# ===============================

			set options($optmain) [list $argre $argtype]

			foreach opt $optnames {
				# Check for duplicate option definitions
				# ======================================
				
				if [info exists refoptions($opt)] {
					Juf::Error::fatal $opt "Duplicate option definition"
				}

				# replace dashes with underscores
				# ===============================

				regsub -all -- - $opt _ opt

				set refoptions($opt) $optmain

				if {$opt == ""} {
					Juf::Error::fatal "\"$optspec\"" "Invalid option specification"
				}

				# Leading dashes aren't permitted
				# ===============================
				
				if {[string index $opt 0] == "-"} {
					Juf::Error::fatal "\"$optspec\"" "Invalid option specification"
				}
			}
		} else {
			Juf::Error::fatal "\"$optspec\"" "Invalid option specification"
		}
	}
		
	# ------------------------------
	# process command line arguments
	# ------------------------------

	set loop 0
	set max [llength $oldargv]
	while {$loop < $max} {
		set arg [lindex $oldargv $loop]
		if [regexp -- "^(-+)(\[^=\]*)(=(.*))?" \
				[lindex $oldargv $loop] ignore dashes name rmd value] {
			
			# replace dashes with underscores
			# -------------------------------

			regsub -all -- - $name _ optmain
			
			if [info exists refoptions($optmain)] {
				if {$dashes == "-"} {
					if {$rmd != ""} {
						# -@OPTCHAR@TEXT not allowed
						Juf::Error::fatal "$name$rmd" "invalid option"
					}
					set argre [lindex $options($refoptions($optmain)) 0]
					if {$argre == "="} {
						incr loop
						if {$loop == $max} {
							Juf::Error::fatal $name "option requires an argument"
						}
						set opts($refoptions($optmain)) [lindex $oldargv $loop]
					} else {
						set opts($refoptions($optmain)) ""
					}
				} else {
					set opts($refoptions($optmain)) $value
				}
			} else {
				Juf::Error::fatal $name "invalid option"
			}
		} else {
			# This is an argument; store it in the new array
			# ==============================================

			lappend new_argv $arg
		}
		incr loop
	}

	set resargv $new_argv
}

# ------------------------------------------------------
# juf_getopts_listspecs - list options
#
# in: 	$optspecs	list with option specifications 
# 		$outid		file identifier for resulting output
# ------------------------------------------------------

proc juf_getopts_listspecs {optspecs {outid stdout}} {
	global juf_getopts_initskip juf_getopts_argdescrpos
	set optarr('') ""
	unset optarr('')
	set optrefarr('') ""
	unset optrefarr('')

	foreach optspec $optspecs {

		set spec [lindex $optspec 0]
		set shortref [lindex $optspec 2]
		set longref [lindex $optspec 1]

		# Look for argument specifiers
		# ============================

		if [regexp {^([^=:]+)(([=:])([s]))?$} $spec dummy optnames dummy argre argtype] {
			set optnames [split $optnames "|"]
			set shortopts {}
			set longopts {}
			set optbuf {}

			# Divide option names into long and short options
			# ===============================================

			foreach opt $optnames {
				
				# Check for duplicate option names in 
				# the current option specification
				
				if {[lsearch -exact $optbuf $opt] >= 0} {
					Juf::Error::warning $opt "Duplicate option definition"
					continue
				}
				lappend optbuf $opt

				if {$opt == ""} {
					Juf::Error::fatal "\"$optspec\"" "Invalid option specification"
				} elseif {[string length $opt] == 1} {
					lappend shortopts $opt
				} else {
					lappend longopts $opt
				}
				# Leading dashes aren't permitted
				# ===============================
				
				if {[string index $opt 0] == "-"} {
					Juf::Error::fatal "\"$optspec\"" "Invalid option specification"
				}
				# Check for duplicate definitions
				# ===============================

				if [info exists optrefarr($opt)] {
					Juf::Error::fatal $opt "Duplicate option definition"
				}
			}
		} else {
q			Juf::Error::fatal "\"$optspec\"" "Invalid option specification"
		}
		
		# Seek the sort key 
		# =================
			
		if {[llength $shortopts] > 0} {
			set sortkey [lindex $shortopts 0]
		} else {
			set sortkey [lindex $longopts 0]
		}

		# Generate entry for option array
		# ===============================

		set optarr($sortkey) [list $shortopts $longopts $shortref $longref $argre]
		# Generate references to option array entry
		# =========================================

		foreach opt $optnames {
			set optrefarr($opt) $sortkey
		}
	}

	# List options
	# ============

	foreach sortkey [lsort -command {
		proc sort {a b} {
			set ret [string compare [string tolower $a] [string tolower $b]]
			if {$ret == 0} {string compare $b $a} else {set ret}
		}
		sort } [array names optarr]] {
		set shortopts [lindex $optarr($sortkey) 0]
		set longopts [lindex $optarr($sortkey) 1]
		set shortref [lindex $optarr($sortkey) 2]
		set longref [lindex $optarr($sortkey) 3]
		set argre [lindex $optarr($sortkey) 4]
		set line {}

		foreach opt $shortopts {
			set buf "-$opt"
			if {$shortref != ""} {
				if {$argre == ":"} {	
					# optional argument
					append buf " \[$shortref\]"
				} else {
					append buf " $shortref"
				}
			}
			lappend line $buf
		}

		foreach opt $longopts {
			set buf "--$opt"
			if {$shortref != ""} {
				if {$argre == ":"} {	
					# optional argument
					append buf "\[=$shortref\]"
				} else {
					append buf "=$shortref"
				}
			}
			lappend line $buf
		}

		set buf [format "%${juf_getopts_initskip}s%-s" "" [join $line ", "]]

		if {$longref != ""} {
			if {[string length $buf] > [expr $juf_getopts_argdescrpos - 2]} {

				# write argument description on an extra line
				# ===========================================

				puts $outid $buf
				set buf ""
			}
			puts $outid [format "%-[expr $juf_getopts_argdescrpos - 1]s%s" $buf $longref]
		} else {
			puts $outid $buf
		}
	}
}

package provide juf::getopts 0.0.1