# This is a -*- Tcl -*- library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# list.tcl --- Functions for manipulating lists
#
# Copyright (C) 1997, 1998 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0.4
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package require Jufbase
package require Error

namespace eval Juf {namespace eval Sequence {}}

# -----------------------------------------------------
# FUNCTION: append [OPTION ...] NAME [VALUE ...]
#
# Works like the Tcl builtin `lappend', but considers
# these options:
#
# -nonempty		Append only non-empty values.
# --			End of options.
# -----------------------------------------------------

proc Juf::Sequence::append {args} {
	while [llength $args] {
		switch -exact -- [lindex $args 0] {
			-nonempty {
				set nonempty 1
				set args [lrange $args 1 end]
			}
			-- {
				set args [lrange $args 1 end]
				break
			}
			default break
		}
	}
	#
	# at least one argument, the list name, is needed
	#
	if ![llength $args] {
		Juf::Error::fault wrongargs \
				"Juf::Sequence::append varName ?value value ...?"
	}
	upvar [Juf::Sequence::shift args] name
	#
	# no options specified => use builtin
	#
	if ![info exists nonempty] {return [eval lappend name $args]}
	#
	# loop and append only nonempty values
	#
	foreach value $args {
		if [string compare $value ""] {
			lappend name $value
		}
	}
	if [info exists name] {return $name}
}

Juf::deprecate juf_seqappend Juf::Sequence::append

# --------------------------------------------------------------------
# FUNCTION: shift NAME [COUNT]
#
# Removes COUNT elements from the list stored in the variable NAME
# and returns the last element removed. Default for COUNT is 1.
# --------------------------------------------------------------------

proc Juf::Sequence::shift {name {count 1}} {
	upvar $name list
	set retel [lindex $list [expr $count - 1]]
	set list [lrange $list $count end]
	return $retel
}

Juf::deprecate juf_seqshift Juf::Sequence::shift

# ------------------------------------------------------------------
# FUNCTION: pop NAME [COUNT]
#
# Removes COUNT elements from the end of the list stored in the
# variable NAME and returns the last element removed.
# Default for COUNT is 1.
# ------------------------------------------------------------------

proc Juf::Sequence::pop {name {count 1}} {
	upvar $name list
	set pos [expr [llength $list] - $count]
	set retel [lindex $list $pos]
	set list [lreplace $list $pos end]
	return $retel
}

Juf::deprecate juf_seqpop Juf::Sequence::pop

# --------------------------------------------------------------
# FUNCTION: assign LIST [NAME ...]
#
# Sets value of the variables specified by the NAME arguments to
# that of the existing list elements. Returns remaining list
# elements. If the number of variables exceeds the list length,
# the remaining variables will be removed.
# --------------------------------------------------------------

proc Juf::Sequence::assign {list args} {
	for {set i 0} {$i < [llength $list]} {incr i} {
		if ![llength $args] {
			return [lrange $list $i end]
		}
		upvar [Juf::Sequence::shift args] name
		set name [lindex $list $i]
	}
	while {[llength $args]} {
		upvar [Juf::Sequence::shift args] name
		if [info exists name] {unset name}
	}
}

Juf::deprecate juf_seqassign Juf::Sequence::assign

package provide Sequence 0.0.4
package provide juf::list 0.0.1
