# This is a -*- Tcl -*- library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# string.tcl --- String manipulation functions
#
# Copyright (C) 1996, 1997, 1998 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0.4
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package require Jufbase
namespace eval Juf {namespace eval String {}}

# ----------------------------------------------------------------------
# PROC: juf_reformat FMT REGEXP NAME [ARG ...]
#
# Returns a string derived from FMT with all matches to REGEXP replaced.
# ----------------------------------------------------------------------

proc juf_reformat {fmt regexp name args} {
	upvar $name opts

	while {[regexp -indices -- $regexp $fmt area token]} {
		# area of string to be replaced
		set first [lindex $area 0]
		set last [lindex $area 1]
		# add part of string before match to result
		append result [string range $fmt 0 [expr $first - 1]]
		# check if specifier is valid
		set fspec [eval string range {$fmt} $token]
		if [info exists opts($fspec)] {
			# O.K., replace with result of command
			append result [uplevel $opts($fspec) $args]
		} else {
			error "bad field specifier \"$fspec\""
		}
		# continue with regex matching
		set fmt [string range $fmt [expr $last + 1] end]
	}
	append result $fmt
}

# -------------------------------------------------------
# PROC: juf_string_count STRING [ARRNAME]
#
# Returns the number of different characters in STRING.
# If ARRNAME is given, this function generates a new
# array ARRNAME with characters as keys and the character
# count as values.
# -------------------------------------------------------

proc juf_string_count {string {arrname ""}} {
	set v_len [string length $string]
	if [string length $arrname] {
		upvar $arrname v_arr
		if [info exists v_arr] {
			unset v_arr
		}
	}
	for {set i 0} {$i < $v_len} {incr i} {
		if [info exists v_arr([string index $string $i])] {
			incr v_arr([string index $string $i])
		} else {
			set v_arr([string index $string $i]) 1
		}
	}
	return [array size v_arr]
}

# -----------------------------------------------------------
# PROC: compose STRING NUMBER
#
# Composes a string of NUMBER times of STRING.
# -----------------------------------------------------------

proc Juf::String::compose {string number} {
	set buf ""
	for {set i 0} {$i < $number} {incr i} {
		append buf $string
	}
	return $buf
}

Juf::deprecate juf_compose Juf::String::compose

# ---------------------------------------------------
# juf_strcasecmp - case insensitive string comparison
#
# in:	string1, string2	strings to compare
# out:	similar to `string compare'
# ---------------------------------------------------

proc juf_strcasecmp {string1 string2} {
	string compare [string tolower $string1] [string tolower $string2]
}

# --------------------------------------------------------------------------
# juf_split [SWITCHES] STRING [EXP LIMIT]
#
# Returns a list created by splitting STRING at each match
# of the regular expression EXP. The matches are not included in the list
# If EXP is omitted  
# or an empty string is given, it defaults to whitespace. If LIMIT
# is given, it stops the matching process, so that the size of the resulting
# list is equal or lesser than LIMIT.
# If invoked with the `-preserve' switch, the matches are added to the list.
# If invoked with the `-showempty' switch, an empty element is added to
# the list for each match.
# --------------------------------------------------------------------------

proc juf_split {args} {
	if ![llength $args] {
		error "wrong # args: should be \"juf_split \"?options? string ?exp? ?limit?"
	}
	#
	# check for switches
	#
	set showempty 0
	set preserve 0
	while 1 {
		switch -exact -- [lindex $args 0] {
			-preserve {
				set preserve 1
				set args [lrange $args 1 end]
			}
			-showempty {
				set showempty 1
				set args [lrange $args 1 end]
			} -- {
				set args [lrange $args 1 end]
				break
			}
			default break
		}
	}
	if {![llength $args] || [llength $args] > 3} {
		error "wrong # args: should be \"juf_split \"?options? string ?exp? ?limit?"
	}
	if {$showempty && $preserve} {
		error "conflicting switches: -showempty and -preserve"
	}
	#
	# default regular expression (should match a sequence of whitespace)
	#
	set exp [lindex $args 1]
	if {$exp == ""} {
		set exp "\[\t\r\n ]+"
	}
	#
	# initializations
	#
	set limit [lindex $args 2]
	set string [lindex $args 0]
	if {$limit != "" && $limit <= 0} {return ""}
	set v_rmd $string
	set v_list ""
	#
	# loop through
	#
	while {[regexp -indices -- $exp $v_rmd v_res]} {
		#
		# maximum number of elements reached ?
		#
		if {$limit != ""} {
			if {[expr [llength $v_list] + 1] == $limit} {
				lappend v_list $v_rmd
				return $v_list 
			}
		}
		#
		# if the result of the match is an empty string,
		# regexp returns (0 -1)
		#
		if {[lindex $v_res 1] == -1} {
			if [string length $v_rmd] {
				if $showempty {
					lappend v_list ""
				}
				lappend v_list [string index $v_rmd 0]
				set v_rmd [string range $v_rmd 1 end]
				continue
			} else {
				return $v_list
			}
		}
		#
		# if start of string doesn't match, we
		# have an element of the resulting list
		#
		if {[lindex $v_res 0] != 0} {
			lappend v_list [string range $v_rmd 0 \
					[expr [lindex $v_res 0] - 1]]
		}
		if $showempty {
			lappend v_list ""
			if {$limit != ""} {
				if {[llength $v_list] == $limit} {
					return $v_list 
				}
			}
		}
		if $preserve {
			lappend v_list [string range $v_rmd [lindex $v_res 0] \
					[lindex $v_res 1]]
			if {$limit != ""} {
				if {[llength $v_list] == $limit} {
					return $v_list 
				}
			}
		}
		#
		# create and check remainder of string
		#
		set v_rmd [string range $v_rmd [expr [lindex $v_res 1] + 1] end]
		if {$v_rmd == ""} {return $v_list}
	}
	#
	# add remainder of string to result
	#
	lappend v_list $v_rmd
}

package provide String 0.0.4

# SOME SETTINGS FOR GOOD OLD EMACS:
# Local Variables:
# mode: tcl
# tcl-default-application: "tclsh"
# End:

